@@ -10,13 +10,31 @@ from bluepy.btle import DefaultDelegate
 _LOGGER = logging.getLogger(__name__)
 
 
+def retry(retries: int = 3):
+    def decor(f: Callable):
+        def wrapper(*args, **kwargs):
+            for i in range(retries):
+                try:
+                    _LOGGER.debug("Trying %d/%d: %s(args=%s,kwargs=%s)", i, retries, f.__name__, args, kwargs)
+                    return f(*args, **kwargs)
+                except Exception as e:
+                    next_message = "Will try again" if i < retries else "Will not try again"
+                    _LOGGER.warning("Got exception: %s. %s", str(e), next_message)
+                    pass
+            else:
+                _LOGGER.critical("Retry limit (%d) exceeded for %s(%s, %s)", retries, f.__name__, args, kwargs)
+
+        return wrapper
+    return decor
+
+
 class TionDelegation(DefaultDelegate):
     def __init__(self):
-        self._data: List = []
+        self._data: List[bytearray] = []
         self.__topic = None
         DefaultDelegate.__init__(self)
 
-    def handleNotification(self, handle: int, data: bytes):
+    def handleNotification(self, handle: int, data: bytearray):
         self._data.append(data)
         _LOGGER.debug("Got data in %d response %s", handle, bytes(data).hex())
         try:
@@ -28,7 +46,7 @@ class TionDelegation(DefaultDelegate):
         self.__topic = topic
 
     @property
-    def data(self) -> bytes:
+    def data(self) -> bytearray:
         return self._data.pop(0)
 
     @property
@@ -102,6 +120,7 @@ class tion(TionDummy):
         self.__failed_connects: int = 0
         self.__connections_count: int = 0
         self.__notifications_enabled: bool = False
+        self.have_breezer_state: bool = False
 
         if self.mac == "dummy":
             _LOGGER.warning("Dummy mode detected!")
@@ -144,14 +163,6 @@ class tion(TionDummy):
         """
         pass
 
-    @abc.abstractmethod
-    def _get_data_from_breezer(self) -> bytearray:
-        """ Get byte array with brezer response on state request
-        Returns:
-          breezer response
-        """
-        raise NotImplementedError()
-
     @abc.abstractmethod
     def _generate_model_specific_json(self) -> dict:
         """
@@ -220,6 +231,7 @@ class tion(TionDummy):
         """
         try:
             self.connect()
+            self._try_write(request=self.command_getStatus)
             response = self._get_data_from_breezer()
         finally:
             if not keep_connection:
@@ -329,40 +341,13 @@ class tion(TionDummy):
 
     def _disconnect(self):
         if self.connection_status != "disc":
-            if self.__notifications_enabled:
-                self._disable_notifications()
-
             self._btle.disconnect()
 
+    @retry(retries=3)
     def _try_write(self, request: bytearray):
         _LOGGER.debug("Writing %s to %s", bytes(request).hex(), self.write.uuid)
         return self.write.write(request)
 
-    def _do_action(self, action: Callable, max_tries: int = 3, *args, **kwargs):
-        tries: int = 0
-        while tries < max_tries:
-            _LOGGER.debug("Doing " + action.__name__ + ". Attempt " + str(tries + 1) + "/" + str(max_tries))
-            try:
-                response = action(*args, **kwargs)
-                break
-            except Exception as e:
-                tries += 1
-                _LOGGER.warning("Got exception while " + action.__name__ + ": " + str(e))
-                pass
-        else:
-            if action.__name__ == '_connect':
-                message = "Could not connect to " + self.mac
-            elif action.__name__ == '__try_write':
-                message = "Could not write request + " + kwargs['request'].hex()
-            elif action.__name__ == '__try_get_state':
-                message = "Could not get updated state"
-            else:
-                message = "Could not do " + action.__name__
-
-            raise TionException(action.__name__, message)
-
-        return response
-
     def __write_to_notify_handle(self, data):
         need_response: bool = True if self.model == "Lite" else False
         _LOGGER.debug("Notify handler is %s", self.notify.getHandle())
@@ -565,3 +550,64 @@ class tion(TionDummy):
         self.__connections_count -= 1
         if self.__connections_count <= 0:
             self._disconnect()
+
+    @property
+    @abc.abstractmethod
+    def command_getStatus(self) -> bytearray:
+        raise NotImplementedError()
+
+    @abc.abstractmethod
+    def _collect_message(self, package: bytearray) -> bool:
+        """
+        Collects message from several package
+        Must set self._data
+
+        :param package: single package from breezer
+        :return: Have we full response from breezer or not
+        """
+        raise NotImplementedError()
+
+    @property
+    @abc.abstractmethod
+    def _dummy_data(self) -> bytearray:
+        """
+        full response from breezer for test, as it was collected from packages: without deader and CRC
+        """
+        raise NotImplementedError()
+
+    def _get_data_from_breezer(self) -> bytearray:
+        """ Get byte array with breezer response on state request
+
+        :returns:
+          breezer response
+        """
+        self.have_breezer_state = False
+
+        _LOGGER.debug("Collecting data")
+
+        i = 0
+
+        while i < 10:
+            if self.mac == "dummy":
+                return self._dummy_data
+            else:
+                if self._delegation.haveNewData:
+                    byte_response = self._delegation.data
+                    if self._collect_message(byte_response):
+                        self.have_breezer_state = True
+                        break
+                    i = 0
+                else:
+                    self._btle.waitForNotifications(1.0)
+                i += 1
+        else:
+            _LOGGER.debug("Waiting too long for data")
+            self.notify.read()
+
+        if self.have_breezer_state:
+            result = self._data
+
+        else:
+            raise TionException("_get_data_from_breezer", "Could not get breezer state")
+
+        return result
\ No newline at end of file
